package org.xbib.groovy.crypt

import java.nio.charset.StandardCharsets
import java.security.MessageDigest
import java.security.NoSuchAlgorithmException

/**
 * MD5 crypt implementation.
 */
class Md5Crypt extends Crypt {

    private static final String SALT_PREFIX = '$1$'

    private static final int MAX_SALT_LENGTH = 8

    private static final int ROUNDS = 1000

    /**
     * Constructs a new instance.
     * @param type
     */
    Md5Crypt(CryptType type) {
        super(type)
    }

    @Override
    protected String doCrypt(String password, Salt salt)
            throws NoSuchAlgorithmException, UnsupportedEncodingException {
        byte[] encrypted = doCrypt(password.getBytes(StandardCharsets.UTF_8),
                salt.getBytes(MAX_SALT_LENGTH, StandardCharsets.UTF_8))
        return passwordToString(encrypted, salt, MAX_SALT_LENGTH)
    }

    protected byte[] doCrypt(byte[] password, byte[] salt)
            throws NoSuchAlgorithmException, UnsupportedEncodingException {
        final MessageDigest a = type.newDigest()
        final int digestLength = a.getDigestLength()
        a.update(password)
        a.update(SALT_PREFIX.getBytes(StandardCharsets.UTF_8))
        a.update(salt)
        final MessageDigest b = type.newDigest()
        b.update(password)
        b.update(salt)
        b.update(password)
        final byte[] sumB = b.digest()
        int max = (password.length / digestLength) as int
        for (int i = 0; i < max; i++) {
            a.update(sumB)
        }
        a.update(sumB, 0, password.length % digestLength)

        /* The original implementation now does something weird: for every 1
         * bit in the key the first 0 is added to the buffer, for every 0 bit
         * the first character of the key.  This does not seem to be what was
         * intended but we have to follow this to be compatible.
         */
        final byte[] zero = [0]
        for (int length = password.length; length != 0; length >>>= 1) {
            if ((length & 1) != 0) {
                a.update(zero)
            } else {
                a.update(password, 0, 1)
            }
        }
        final byte[] sumA = a.digest()
        byte[] ac = sumA
        for (int i = 0; i < ROUNDS; i++) {
            final MessageDigest c = type.newDigest()
            if (i % 2 != 0) {
                c.update(password)
            } else {
                c.update(ac)
            }
            if (i % 3 != 0) {
                c.update(salt)
            }
            if (i % 7 != 0) {
                c.update(password)
            }
            if (i % 2 != 0) {
                c.update(ac)
            } else {
                c.update(password)
            }
            ac = c.digest()
        }
        ac
    }

    @Override
    protected String encodeParameters(Object... params) {
        throw new UnsupportedOperationException()
    }

    @Override
    protected String encodePassword(byte[] password) {
        StringBuilder sb = new StringBuilder()
        sb.append(Encoder.base64(password[0], password[6], password[12], 4))
        sb.append(Encoder.base64(password[1], password[7], password[13], 4))
        sb.append(Encoder.base64(password[2], password[8], password[14], 4))
        sb.append(Encoder.base64(password[3], password[9], password[15], 4))
        sb.append(Encoder.base64(password[4], password[10], password[5], 4))
        sb.append(Encoder.base64((byte) 0, (byte) 0, password[11], 2))
        sb.toString()
    }

}
