package org.xbib.groovy.crypt

import java.nio.charset.Charset

/**
 * Salt for encryption.
 */
class Salt {

    String type

    String params

    String text

    Salt(String salt) {
        if (salt.charAt(0) != '$') {
            this.type = "0"
            this.params = null
            this.text = salt
            return
        }
        int index = 1
        int extent = salt.indexOf('$', index)
        if (extent == -1) {
            throw new IllegalArgumentException("illegal salt format")
        }
        this.type = salt.substring(index, extent)
        index = extent + 1
        if (index > salt.length()) {
            throw new IllegalArgumentException("illegal salt format")
        }
        extent = salt.indexOf('$', index)
        if (extent == -1 || salt.substring(index, extent).indexOf('=') == -1) {
            this.params = null
        } else {
            this.params = salt.substring(index, extent)
            index = extent + 1
            if (index > salt.length()) {
                throw new IllegalArgumentException("illegal salt format")
            }
        }
        extent = salt.indexOf('$', index)
        if (extent == -1) {
            extent = salt.length()
        }
        this.text = salt.substring(index, extent)
    }

    /**
     * Gets the salt text truncated to a given maximum length.
     * @param maxLength maximum length
     * @return truncated salt text
     */
    String getText(int maxLength) {
        text.substring(0, Math.min(text.length(), maxLength))
    }

    /**
     * Gets the salt text as an array of bytes of a given character encoding.
     * @param charset character set name
     * @return byte array
     * @throws UnsupportedEncodingException
     */
    byte[] getBytes(int maxLength, Charset charset)
            throws UnsupportedEncodingException {
        try {
            ByteArrayOutputStream outputStream = new ByteArrayOutputStream()
            Writer writer = new OutputStreamWriter(outputStream, charset)
            writer.write(text, 0, Math.min(text.length(), maxLength))
            writer.close()
            return outputStream.toByteArray()
        } catch (IOException ex) {
            throw new RuntimeException(ex)
        }
    }

}