package org.xbib.groovy.crypt.random

import java.security.SecureRandom

/**
 * <p>{@link SeedGenerator} implementation that uses
 * {@link java.security.SecureRandom} to generate random seed data.</p>
 *
 * <p>The advantage of using SecureRandom for seeding but not as the
 * primary random number generator is that we can use it to seed random number generators
 * that are much faster than SecureRandom.</p>
 *
 * <p>This is the only seeding strategy that is guaranteed to work on all
 * platforms and therefore is provided as a fall-back option should
 * none of the other provided {@link SeedGenerator} implementations be
 * useable.</p>
 */

class SecureRandomSeedGenerator implements SeedGenerator {
    private static final SecureRandom INSTANCE = new SecureRandom()

    byte[] generateSeed(int length) throws SeedException {
        return INSTANCE.generateSeed(length)
    }
}