package org.xbib.groovy.git.operation

import java.util.concurrent.Callable

import org.xbib.groovy.git.Repository
import org.xbib.groovy.git.annotations.Operation
import org.xbib.groovy.git.service.ResolveService
import org.eclipse.jgit.api.DescribeCommand

/**
 * Find the nearest tag reachable. Returns an {@link String}}.
 */
@Operation('describe')
class DescribeOp implements Callable<String> {
    private final Repository repo

    DescribeOp(Repository repo){
        this.repo = repo
    }

    /**
     * Sets the commit to be described. Defaults to HEAD.
     * @see {@link ResolveService#toRevisionString(Object)}
     */
    Object commit

    /**
     * Whether to always use long output format or not.
     */
    boolean longDescr

    /**
     * Include non-annotated tags when determining nearest tag.
     */
    boolean tags

    /**
     * glob patterns to match tags against before they are considered
     */
    List<String> match = []

    String call(){
        DescribeCommand cmd = repo.jgit.describe()
        if (commit) {
            cmd.setTarget(new ResolveService(repo).toRevisionString(commit))
        }
        cmd.setLong(longDescr)
        cmd.setTags(tags)
        if (match) {
            cmd.setMatch(match as String[])
        }
        return cmd.call()
    }
}
